/*****************************************************************************

  FileName:	5ESSmes.c

  Contents:	Pack/Unpack functions. These functions will unpack a 5ESS ISDN
		message from the bit packed original format into structs
		that contains variables sized by the user. It will also pack
		the struct back into a Q.931 message as required.

		See 5ESS.h for description. 

  License/Copyright:

  Copyright (c) 2007, Jan Vidar Berger, Case Labs, Ltd. All rights reserved.
  email:janvb@caselaboratories.com  

  Copyright (c) 2007, Michael Jerris. All rights reserved.
  email:mike@jerris.com
  
  Copyright (c) 2007, Michael S. Collins, All rights reserved.
  email:mcollins@fcnetwork.com

  Redistribution and use in source and binary forms, with or without 
  modification, are permitted provided that the following conditions are 
  met:

	* Redistributions of source code must retain the above copyright notice, 
	this list of conditions and the following disclaimer.
	* Redistributions in binary form must reproduce the above copyright notice, 
	this list of conditions and the following disclaimer in the documentation 
	and/or other materials provided with the distribution.
	* Neither the name of the Case Labs, Ltd nor the names of its contributors 
	may be used to endorse or promote products derived from this software 
	without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
  POSSIBILITY OF SUCH DAMAGE.

*****************************************************************************/

#define Q931PRIVATE

#include "5ESS.h"

/*
 *
 */
struct Q931MessageIE ATT5ESSMessageIEs[] = {
	/*
	 * AT&T 5ESS IE table
	 */

	/*****************************************
	 * Messages with Protocol Discriminator 8
	 *****************************************/

	/* ALERTING */
	{ Q931mes_ALERTING, 8, {
			{ Q931ie_CHANNEL_IDENTIFICATION,   4,  8, Q931_IE_CODESET_0, Q931_IEF_TO_BOTH },
			{ Q931ie_REDIRECTING_NUMBER,       3, 20, Q931_IE_CODESET_0, Q931_IEF_TO_BOTH },
			{ Q931ie_USER_USER,                3,131, Q931_IE_CODESET_0, Q931_IEF_TO_BOTH },

			/* Network / User IEs */
			{ 0, 0, 0, 0 },
	}},

	/* CALL PROCEEDING */
	{ Q931mes_CALL_PROCEEDING, 8, {
			{ Q931ie_CHANNEL_IDENTIFICATION,   4,  8, Q931_IE_CODESET_0, Q931_IEF_TO_BOTH | Q931_IEF_MANDATORY 	},
			{ 0, 0, 0, 0 },
	}},

	/* CONGESTION CONTROL */
	{ Q931mes_CONGESTION_CONTROL, 8, {
			{ Q931ie_CONGESTION_LEVEL, 1,  0, Q931_IE_CODESET_0, Q931_IEF_TO_BOTH | Q931_IEF_MANDATORY },
			{ Q931ie_CAUSE,            4, 22, Q931_IE_CODESET_0, Q931_IEF_TO_BOTH | Q931_IEF_MANDATORY },
			{ 0, 0, 0, 0 },
	}},

	/* CONNECT */
	{ Q931mes_CONNECT, 8, {
			{ Q931ie_CONNECTED_NUMBER,         4,  8, Q931_IE_CODESET_0, Q931_IEF_TO_BOTH },
			{ Q931ie_CHANNEL_IDENTIFICATION,   3, 18, Q931_IE_CODESET_0, Q931_IEF_TO_NET  },
			{ Q931ie_REDIRECTING_NUMBER,       3, 20, Q931_IE_CODESET_0, Q931_IEF_TO_BOTH },
			{ Q931ie_USER_USER,                3,131, Q931_IE_CODESET_0, Q931_IEF_TO_BOTH },

			/* Network / User IEs */
			{ 0, 0, 0, 0 },
	}},

	/* CONNECT ACKNOWLEDGE */
	{ Q931mes_CONNECT_ACKNOWLEDGE, 8, {
			{ 0, 0, 0, 0 },
	}},

	/* DISCONNECT */
	{ Q931mes_DISCONNECT, 8, {
			{ Q931ie_CAUSE,              4, 22, Q931_IE_CODESET_0, Q931_IEF_TO_BOTH | Q931_IEF_MANDATORY },
			{ Q931ie_USER_USER,          3,131, Q931_IE_CODESET_0, Q931_IEF_TO_BOTH },

			/* Network / User IEs */

			{ 0, 0, 0, 0 },
	}},

	/* PROGRESS */
	{ Q931mes_PROGRESS, 8, {
			{ Q931ie_CAUSE,                    4, 22, Q931_IE_CODESET_0, Q931_IEF_TO_BOTH },
			{ Q931ie_CHANNEL_IDENTIFICATION,   3, 18, Q931_IE_CODESET_0, Q931_IEF_TO_NET  },
			{ Q931ie_PROGRESS_INDICATOR,       4,  0, Q931_IE_CODESET_0, Q931_IEF_TO_BOTH | Q931_IEF_MANDATORY },
			{ 0, 0, 0, 0 },
	}},

	/* RELEASE */
	{ Q931mes_RELEASE, 8, {
			{ Q931ie_CAUSE,   4, 22, Q931_IE_CODESET_0, Q931_IEF_TO_BOTH },
			{ 0, 0, 0, 0 },
	}},

	/* RELEASE COMPLETE */
	{ Q931mes_RELEASE_COMPLETE, 8, {
			{ Q931ie_CAUSE,   4, 22, Q931_IE_CODESET_0, Q931_IEF_TO_BOTH },
			{ 0, 0, 0, 0 },
	}},

	/* SETUP */
	{ Q931mes_SETUP, 8, {
			{ Q931ie_BEARER_CAPABILITY,           4,  8, Q931_IE_CODESET_0, Q931_IEF_TO_BOTH | Q931_IEF_MANDATORY },
			{ Q931ie_CHANNEL_IDENTIFICATION,      4,  8, Q931_IE_CODESET_0, Q931_IEF_TO_BOTH | Q931_IEF_MANDATORY },
			{ Q931ie_NETWORK_SPECIFIC_FACILITIES, 4,  9, Q931_IE_CODESET_0, Q931_IEF_TO_BOTH },
			{ Q931ie_CALLING_PARTY_NUMBER,        3, 19, Q931_IE_CODESET_0, Q931_IEF_TO_BOTH },
			{ Q931ie_CALLING_PARTY_SUBADDRESS,    3, 23, Q931_IE_CODESET_0, Q931_IEF_TO_BOTH },
			{ Q931ie_CALLED_PARTY_NUMBER,         3, 20, Q931_IE_CODESET_0, Q931_IEF_TO_BOTH | Q931_IEF_MANDATORY },
			{ Q931ie_CALLED_PARTY_SUBADDRESS,     3, 23, Q931_IE_CODESET_0, Q931_IEF_TO_BOTH },
			{ Q931ie_REDIRECTING_NUMBER,          3, 20, Q931_IE_CODESET_0, Q931_IEF_TO_BOTH },
			{ Q931ie_TRANSIT_NETWORK_SELECTION,   7,  0, Q931_IE_CODESET_0, Q931_IEF_TO_NET },
			{ Q931ie_LOW_LAYER_COMPATIBILITY,     3, 16, Q931_IE_CODESET_0, Q931_IEF_TO_BOTH },
			{ Q931ie_HIGH_LAYER_COMPATIBILITY,    3,  5, Q931_IE_CODESET_0, Q931_IEF_TO_BOTH },
			{ Q931ie_USER_USER,                   3,131, Q931_IE_CODESET_0, Q931_IEF_TO_BOTH },

			/* Network / User defined IEs */
			{ Q931ie_FACILITY,                    3,131, Q931_IE_CODESET_6, Q931_IEF_TO_BOTH },
/*			{ Q931ie_GENERIC_BILLING_DATA,        3,131, Q931_IE_CODESET_6, Q931_IEF_TO_BOTH }, */
/*			{ Q931ie_ITEM,                        3,  0, Q931_IE_CODESET_6, Q931_IEF_TO_BOTH }, */
/*			{ Q931ie_ORIGINATING_ACCESS_SERVICE_TYPE, 4,  0, Q931_IE_CODESET_6, Q931_IEF_TO_BOTH }, */
/*			{ Q931ie_ORIGINATING_LINE_INFORMATION,    3,  0, Q931_IE_CODESET_6, Q931_IEF_TO_BOTH }, */
/*			{ Q931ie_SPECIFIC_EVENT,                  3,  0, Q931_IE_CODESET_6, Q931_IEF_TO_BOTH }, */
/*			{ Q931ie_USER_ENTERED_CODE,               3,131, Q931_IE_CODESET_6, Q931_IEF_TO_BOTH }, */
			{ 0, 0, 0, 0 },
	}},

	/* STATUS */
	{ Q931mes_STATUS, 8, {
			{ Q931ie_CAUSE,      4, 22, Q931_IE_CODESET_0, Q931_IEF_TO_BOTH | Q931_IEF_MANDATORY },
			{ Q931ie_CALL_STATE, 3,  0, Q931_IE_CODESET_0, Q931_IEF_TO_BOTH | Q931_IEF_MANDATORY },
			{ 0, 0, 0, 0 },
	}},

	/* STATUS ENQUIRY */
	{ Q931mes_STATUS_ENQUIRY, 8, {
			{ 0, 0, 0, 0 },
	}},

	/* USER INFORMATION */
	{ Q931mes_USER_INFORMATION, 8, {
			{ Q931ie_MORE_DATA, 1,  0, Q931_IE_CODESET_0, Q931_IEF_TO_BOTH },
			{ Q931ie_USER_USER, 3,131, Q931_IE_CODESET_0, Q931_IEF_TO_BOTH | Q931_IEF_MANDATORY },
			{ 0, 0, 0, 0 },
	}},

	/* FACILITY */
	{ Q931mes_FACILITY, 8, {
			{ Q931ie_CHANNEL_IDENTIFICATION,      4,  8, Q931_IE_CODESET_0, Q931_IEF_TO_BOTH },
			{ Q931ie_NETWORK_SPECIFIC_FACILITIES, 4,  9, Q931_IE_CODESET_0, Q931_IEF_TO_BOTH },

			/* Codeset 6 IEs */
			{ Q931ie_FACILITY,                    3,131, Q931_IE_CODESET_6, Q931_IEF_TO_BOTH },
			{ 0, 0, 0, 0 },
	}},

	/* FACILITY ACKNOWLEDGE */
	{ Q931mes_FACILITY_ACKNOWLEDGE, 8, {
			{ Q931ie_NETWORK_SPECIFIC_FACILITIES, 4,  9, Q931_IE_CODESET_0, Q931_IEF_TO_BOTH },
			{ Q931ie_CALLING_PARTY_NUMBER,        3, 19, Q931_IE_CODESET_0, Q931_IEF_TO_BOTH },

			/* Codeset 6 IEs */
/*			{ ATT5ESSie_OLI,                      3,  0, Q931_IE_CODESET_6, Q931_IEF_TO_USER }, */
			{ 0, 0, 0, 0 },
	}},

	/* FACILITY REJECT */
	{ Q931mes_FACILITY_REJECT, 8, {
			{ Q931ie_CAUSE,                       4, 22, Q931_IE_CODESET_0, Q931_IEF_TO_BOTH | Q931_IEF_MANDATORY },
			{ Q931ie_NETWORK_SPECIFIC_FACILITIES, 4,  9, Q931_IE_CODESET_0, Q931_IEF_TO_BOTH },
			{ 0, 0, 0, 0 },
	}},

	/* REGISTER */
	{ Q931mes_REGISTER, 8, {
			/* Codeset 6 IEs */
			{ Q931ie_FACILITY,                    3,131, Q931_IE_CODESET_6, Q931_IEF_TO_BOTH },
			{ 0, 0, 0, 0 },
	}},


	/*****************************************
	 * Messages with Protocol Discriminator 3
	 *****************************************/

	/* SERVICE */
	{ Q931mes_SERVICE, 3, {
			{ Q931ie_CHANGE_STATUS,          3, 0, Q931_IE_CODESET_0, Q931_IEF_TO_BOTH | Q931_IEF_MANDATORY },
			{ Q931ie_CHANNEL_IDENTIFICATION, 3, 6, Q931_IE_CODESET_0, Q931_IEF_TO_BOTH | Q931_IEF_MANDATORY },
			{ 0, 0, 0, 0 },
	}},

	/* SERVICE ACKNOWLEDGE */
	{ Q931mes_SERVICE_ACKNOWLEDGE, 3, {
			{ Q931ie_CHANGE_STATUS,          3, 0, Q931_IE_CODESET_0, Q931_IEF_TO_BOTH | Q931_IEF_MANDATORY },
			{ Q931ie_CHANNEL_IDENTIFICATION, 3, 6, Q931_IE_CODESET_0, Q931_IEF_TO_BOTH | Q931_IEF_MANDATORY },
			{ 0, 0, 0, 0 },
	}},


	/* <<< EOT >>> */
	{ 0, 0, {{ 0, 0, 0, 0 }}},
};
